/**********************************************************************************************
 *
 * This file declares the Poisson (approximation to binomial) transformer for computing per-set locality.
 * r' = r.BP
 * BP is a Possion approximation to a generalized stochastic Binomial matrix B.
 * Ref: Appendix B in the paper.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#ifndef _POISSON_H_
#define _POISSON_H_

#include "common.h"

// CAUTION: This transformer has been implemented to work with MAX_DIFF_SET_BITS<=8 (r(2^10)-->r(2^18)) and r_size<=512
// For a wider range, you may need to declare precomputed_exp_inc as "long double"

class per_set_locality_poisson_t {
private:
    double precomputed_v[MAX_R_TGT_SIZE+1];
    double precomputed_exp_inc[MAX_DIFF_SET_BITS+1];
public:
    per_set_locality_poisson_t();
    void transform_r(double r[],double rBP[],unsigned int r_size,unsigned int rBP_size,unsigned int num_diff_set_bits) const;
};

#endif

