/**********************************************************************************************
 *
 * This file implements the base class for the miss-ratio predictors.
 * It includes methods used by predictors for all replacement policies.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#ifndef _PRED_MR_H_
#define _PRED_MR_H_

#include "common.h"
#include "poisson.h"

// S'=2^10 has num_diff_set_bits=0, S'=2^18 has num_diff_set_bits=8

class pred_mr_t {
    protected:
        double r_src[MAX_R_SRC_SIZE];                                  // holds r(2^10)
        unsigned int actual_r_src_size;
        double r_tgt[MAX_DIFF_SET_BITS+1][MAX_R_TGT_SIZE];             // r_tgt[x] holds r(2^(x+MIN_INDEX_BITS)) for target cache configurations, where x=num_diff_set_bits
        unsigned int minA[MAX_DIFF_SET_BITS+1];                        // minA[x] holds min. assoc. value for S'=2^(x+MIN_INDEX_BITS), where x=num_diff_set_bits
        unsigned int maxA[MAX_DIFF_SET_BITS+1];                        // maxA[x] holds max. assoc. value for S'=2^(x+MIN_INDEX_BITS), where x=num_diff_set_bits
        per_set_locality_poisson_t per_set_locality;                   // per_set locality transformer
        double estimated_mr[MAX_DIFF_SET_BITS+1][MAX_ASSOC+1];         // estimated_mr[x][y] holds estimated miss-ratio for S'=2^(x+MIN_INDEX_BITS), A'=y, where x=num_diff_set_bits
    public:
        pred_mr_t();
        void read_r_src(const char *file_path);
        void compute_miss_ratio();
        void print_miss_ratio();
        virtual void reset();
        virtual double compute_miss_ratio_specific(unsigned int num_diff_set_bits,unsigned int assoc)=0;
};

#endif

