/**********************************************************************************************
 *
 * This file implements the child predictor class for NMRU.
 * Ref: Section 4.2 in the paper.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#include <math.h>
#include "pred_nmru.h"
#include "util.h"

pred_nmru_t::pred_nmru_t() {
    reset();
}

void pred_nmru_t::reset() {
    int i;
    for(i=0;i<=MAX_DIFF_SET_BITS;i++)
        d_computed[i]=false;
    pred_mr_t::reset();
}

double pred_nmru_t::compute_miss_ratio_specific(unsigned int num_diff_set_bits,unsigned int assoc) {
    double p, sum;
    int i;
    if(!d_computed[num_diff_set_bits]) {
        compute_d(r_tgt[num_diff_set_bits],d_tgt[num_diff_set_bits],2*maxA[num_diff_set_bits]);
        d_computed[num_diff_set_bits]=true;
    }
    if(assoc==2) {                // \phi(A'=2,NMRU)=\phi(A'=2,LRU)
        p=0;
        for(i=0;i<2;i++)     
            p+=r_tgt[num_diff_set_bits][i];
        return 1-p;
    }
    p=r_tgt[num_diff_set_bits][0];
    // uncomment the code below for faster termination, but possibly less accuracy
//    double p_last=p;
//    double ratio;
    int iter;
    double t[2*MAX_ASSOC];
    // increase the iteration count for possibly greater accuracy, but with more computation time
    for(iter=0;iter<5;iter++) {
        sum=0;
        for(i=0;i<2;i++) {
            t[i]=d_tgt[num_diff_set_bits][i]*(1-p);
            phi[i]=1;
            sum+=r_tgt[num_diff_set_bits][i];                       // compute the dot-product r_tgt.\phi(NMRU). Note that \phi_0=1, \phi_1=1
        }
        for(;i<2*assoc;i++) {
            t[i]=d_tgt[num_diff_set_bits][i]*(1-p);
            phi[i]=exp(-(t[i]-t[1])/(assoc-1));                     // replace by expf if that is faster
            sum+=r_tgt[num_diff_set_bits][i]*phi[i];                // compute the dot-product r_tgt.\phi(NMRU)
        }
        p=sum;
        // uncomment the code below for faster termination, but possibly less accuracy
//        if(p_last) {
//            ratio=p/p_last;
//            if(ratio<1.01) break;
//       }
//        p_last=p;
    }
    return 1-p;
}

