/**********************************************************************************************
 *
 * This file declares the child predictor class for PLRU.
 * Ref: Section 4.1 in the paper.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#ifndef _PLRU_H_
#define _PLRU_H

#include "pred_mr.h"

// The dependence of \phi(PLRU) on S' is due to its dependence on r_tgt (See Equation 13)

class pred_plru_t: public pred_mr_t{
private:
    double plru_factors1[2*MAX_ASSOC][2*MAX_ASSOC];         // holds precomputed coefficients
    double plru_factors2[2*MAX_ASSOC][2*MAX_ASSOC];         // holds precomputed coefficients
    bool plru_computed[MAX_DIFF_SET_BITS+1][MAX_ASSOC+1];   // book-keeping to track that phi for the same target configuration is not computed multipled times
    double plru_phi_4[MAX_DIFF_SET_BITS+1][2*MAX_ASSOC];    // plru_phi_4[x] holds \phi(A'=4,PLRU) for S'=2^(x+MIN_INDEX_BITS), where x=num_diff_set_bits
    double plru_phi_8[MAX_DIFF_SET_BITS+1][2*MAX_ASSOC];    // plru_phi_8[x] holds \phi(A'=8,PLRU) for S'=2^(x+MIN_INDEX_BITS), where x=num_diff_set_bits
    double plru_phi_16[MAX_DIFF_SET_BITS+1][2*MAX_ASSOC];   // plru_phi_16[x] holds \phi(A'=16,PLRU) for S'=2^(x+MIN_INDEX_BITS), where x=num_diff_set_bits
    double plru_phi_32[MAX_DIFF_SET_BITS+1][2*MAX_ASSOC];   // plru_phi_32[x] holds \phi(A'=32,PLRU) for S'=2^(x+MIN_INDEX_BITS), where x=num_diff_set_bits
    void plru_compute_phi_4(unsigned int num_diff_set_bits);
    void plru_compute_phi_8(unsigned int num_diff_set_bits);
    void plru_compute_phi_16(unsigned int num_diff_set_bits);
    void plru_compute_phi_32(unsigned int num_diff_set_bits);
public:
    pred_plru_t();
    void reset();
    double compute_miss_ratio_specific(unsigned int num_diff_set_bits,unsigned int assoc);
}; 

#endif
