/**********************************************************************************************
 *
 * This file implements the child predictor class for RANDOM.
 * Ref: Section 4.2 in the paper.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#include <math.h>
#include "pred_random.h"
#include "util.h"

pred_random_t::pred_random_t() {
    reset();
}

void pred_random_t::reset() {
    int i;
    for(i=0;i<=MAX_DIFF_SET_BITS;i++)
        d_computed[i]=false;
    pred_mr_t::reset();
}

double pred_random_t::compute_miss_ratio_specific(unsigned int num_diff_set_bits,unsigned int assoc) {
    double p, sum, t;
    if(!d_computed[num_diff_set_bits]) {
        compute_d(r_tgt[num_diff_set_bits],d_tgt[num_diff_set_bits],2*maxA[num_diff_set_bits]);
        d_computed[num_diff_set_bits]=true;
    }
    p=r_tgt[num_diff_set_bits][0];
    // uncomment the code below for faster termination, but possibly less accuracy
//    double p_last=p;
//    double ratio;
    int i,iter;
    // increase the iteration count for possibly greater accuracy, but with more computation time
    for(iter=0;iter<5;iter++) {
        phi[0]=1;
        sum=r_tgt[num_diff_set_bits][0];                                // compute the dot-product r_tgt.\phi(RANDOM). Note that \phi_0=1.
        if(assoc==2) {
            int i;
            for(i=1;i<2;i++) {
                t=d_tgt[num_diff_set_bits][i]*(1-p);
                phi[i]=exp(-t/assoc);                                   // replace by expf if that is faster
                sum+=r_tgt[num_diff_set_bits][i]*phi[i];                // compute the dot-product r_tgt.\phi(RANDOM)
            }
            for(;i<2*assoc;i++) {
                phi[i]=phi[i-1]*(1-phi[1]);
                sum+=r_tgt[num_diff_set_bits][i]*phi[i];                // compute the dot-product r_tgt.\phi(RANDOM)
            }
        }
        else {
            for(i=1;i<2*assoc;i++) {
                t=d_tgt[num_diff_set_bits][i]*(1-p);
                phi[i]=exp(-t/assoc);                                   // replace by expf if that is faster
                sum+=r_tgt[num_diff_set_bits][i]*phi[i];                // compute the dot-product r_tgt.\phi(RANDOM)
            }
        }
        p=sum;
        // uncomment the code below for faster termination, but possibly less accuracy
//        if(p_last) {
//            ratio=p/p_last;
//            if(ratio<1.01) break;
//        }
//        p_last=p;
    }
    return 1-p;
}


