/**********************************************************************************************
 *
 * This example shows how to read r and compute r'=r.B to 512 positions.
 * B is a generalized stochastic Binomial matrix.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "util.h"
#include "binomial.h"

#define min(x,y) (((x)<=(y))?(x):(y))

const unsigned int max_r_size=(1<<19);      // URDs can range from 0 to max_r_size-1
double r[max_r_size];
const unsigned int rB_size=512;             // r' is maintained for 512 positions
double rB[rB_size];

void init() {
    unsigned int i;
    for(i=0;i<max_r_size;i++)
        r[i]=0;
    for(i=0;i<rB_size;i++)
        rB[i]=0;
}

void print_rB() {
    unsigned int i;
    for(i=0;i<rB_size;i++) {
        printf("%d %g\n",i,rB[i]);
    }
}

int main() {
    init();
    per_set_locality_binomial_t per_set_locality;
    const char *file_path="example_r_S1.txt";
    unsigned int actual_max_size=read_urd(file_path,r,max_r_size);
    per_set_locality.transform_r(r,rB,actual_max_size,rB_size,10);  // Since r(S=1) is the starting point, S=2^0, S'=2^10, num_diff_set_bits=10-0=10
    print_rB();
    return 0;
}

