/**********************************************************************************************
 *
 * This file declares utilities to read r from a file, compute d from r
 * Ref: Equation 1 in the paper.
 *
 * Author: Rathijit Sen 
 *         University of Wisconsin-Madison
 *         rathijit@cs.wisc.edu
 *
 **********************************************************************************************/

#ifndef _UTIL_H_
#define _UTIL_H_

// Each line of the input file for r must be of the form "i P(URD=i)" with i in increasing order. 
// r_i = P(URD=i) must be in [0,1], \sum {r_i} <= 1.
// Refer to the example files example_r_S1.txt and example_r_S1024.txt.
//
// r_\infty is never explicitly specified in the input file. Whenever needed, e.g., estimating d, it is calculated as 1-\sum {r_i}
// So, it is usually the case that for the input file \sum {r_i} < 1.
// r_\infty does not participate in matrix transformations for per-set locality.

unsigned int read_urd(const char *file_path,double r[],unsigned int max_size);
void compute_d(double r[],double d[],unsigned int d_size);

#endif

